use super::WriteElement;

/// [feTurbulence](https://www.w3.org/TR/SVG11/filters.html#feTurbulenceElement)
#[derive(Debug)]
pub struct Turbulence {
    base_frequency: (f32, f32),
    num_octaves: u16,
    seed: u32,
    stitch_tiles: StitchTiles,
    // attr name: type
    noise_type: NoiseType,
}

impl WriteElement for Turbulence {
    #[allow(clippy::str_to_string, reason = "in macro invocation")]
    fn attrs(&self) -> Vec<quick_xml::events::attributes::Attribute> {
        let mut r = gen_attrs!(
            b"baseFrequency" = format!("{} {}", self.base_frequency.0, self.base_frequency.1)
        );

        if self.num_octaves != 1 {
            r.push(gen_attr!(b"numOctaves" = self.num_octaves));
        }

        if self.seed != 0 {
            r.push(gen_attr!(b"seed" = self.seed));
        }

        if self.stitch_tiles != StitchTiles::NoStitch {
            r.push(gen_attr!(b"stitchTiles" = "stitch"));
        }

        if self.noise_type != NoiseType::Turbulence {
            r.push(gen_attr!(b"type" = "fractalNoise"));
        }

        r
    }

    fn tag_name(&self) -> &'static str {
        "feTurbulence"
    }
}

#[derive(Debug, PartialEq, Eq, Clone, Copy)]
pub enum StitchTiles {
    Stitch,
    NoStitch,
}
#[derive(Debug, PartialEq, Eq, Clone, Copy)]
pub enum NoiseType {
    Turbulence,
    FractalNoise,
}
